#include"graphics.h"
#include"stats.h"
#include"constants.h"
#include<errno.h>
#include<string.h>

static char*    queue_default_style = "dotted";
static char*    queue_normal_style = "filled";

static char*    queue_splittable_color = "black";      //used to be red
static char*    queue_normal_color= "black";           //used to black
static char*    queue_immature_color= "black";         //used to be pink
static char*    queue_mergable_color = "black";        //used to be blue

static char*    frozen_shape = "hexagon";
static char*    normal_shape = "rectangle";

char* lookup_color(int type)
{
    switch(type) {
        case 0:  return "0 0 0.2";
        case 1:  return "1 0.6 0.4";
        case 2:  return "1 0.2 0.6";
        case 3:  return "0.8 1 0.8";
        case 4:  return "0.8 0.6 1";
        case 5:  return "0.8 0.4 0";
        case 6:  return "0.8 0 0.2";
        case 7:  return "0.6 0.8 0.4";
        case 8:  return "0.6 0.4 0.6";
        case 9:  return "0.6 0 0.8";
        case 10: return "0.4 0.8 1";
        case 11: return "0.4 0.6 0";
        case 12: return "0.4 0.2 0.2";
        case 13: return "0.2 1 0.4";
        case 14: return "0.2 0.6 0.6";
        case 15: return "0.2 0.2 0.8";
        case 16: return "0 1 1";
        case 17: return "0 0.8 0";
        case 18: return "0 0.4 0.2";
        case 19: return "0 0 0.4";
    }

    return "0.0 0.0 0.0";
};

int lookup_red(char* color)
{
    float red, green, blue;
    sscanf(color, "%f %f %f", &red, &green, &blue);

    red = red*255;
    return (int)(red);
}

int lookup_green(char* color)
{
    float red, green, blue;
    sscanf(color, "%f %f %f", &red, &green, &blue);

    green = green*255;
    return (int)(green);
}

int lookup_blue(char* color)
{
    float red, green, blue;
    sscanf(color, "%f %f %f", &red, &green, &blue);

    blue = blue*255;
    return (int)(blue);
}

/*
void ptype_key_print(FILE* file)
{
    int i;

//    fprintf(file, "digraph PTYPE_KEY {\n");

    for(i=0; i < PTYPE_SIZE; i++)
    { 
        char* color = lookup_color(i);
        int red =   lookup_red(color);
        int green = lookup_green(color);
        int blue =  lookup_blue(color);

        fprintf(file, "    K%d [ color=\"#%02x%02x%02x\", style=filled, fontcolor=white, label=\"%s\" ]\n", i, red, green, blue, ptype_string(i));
    }

    //fprintf(file, "}\n");
}
*/

int sum(int size, int* array)
{
    int i;
    int ret = 0;
    for(i=0; i < size; i++) ret += array[i];
    return ret;
}

void pie_chart_write(FILE* file, char* color, double traffic, int size, int* pie_data)
{
//    assert(file != NULL && pie_data != NULL);

    int i;
    int total = sum(size, pie_data);
    int wedges = pie_wedges(size, pie_data);
    int radius; 
    int border = 3;
    int length_x, length_y;

    if(traffic <= MAX_MERGE_RATE)
        radius = 20; // Perhaps use MIN_RATE here?
    else if(traffic >= MAX_RATE)
    {
        radius = 50;

        if(traffic >= MAX_RATE+10) radius = 60;
        if(traffic >= MAX_RATE+20) radius = 70; // 
        if(traffic >= MAX_RATE+30) radius = 80;
        if(traffic >= MAX_RATE+40) radius = 90; //
        if(traffic >= 99) radius = 100;
    }
    else
        radius = 30;


    length_x = (radius+border)*2;
    length_y = (radius+border)*2;

    double current_arc = 0;

    if(strcmp(color, "pink") == 0)
        color = "1 0.8 0.8";
    else if(strcmp(color, "grey") == 0)
        color = "0.5 0.5 0.5";
    else if(strcmp(color, "red") == 0)
        color = "1 0.2 0.2";
    else if(strcmp(color, "blue") == 0)
        color = "0.2 0.2 1";
    else
    {
        fprintf(stderr, "UNKNOWN COLOR\n");
        exit(0);
    }



    fprintf(file, "%%%%!PS-Adobe EPSF-3.0\n");
//  The creator and title should go here.\n";
    fprintf(file, "%%%%BoundingBox: %d %d %d %d\n", 0, 0, length_x, length_y);
    fprintf(file, "%%%%EndComments\n");
  
    fprintf(file, "newpath\n%s setrgbcolor\n", color);
    fprintf(file, "0 0 moveto\n0 %d lineto\n %d %d lineto\n%d 0 lineto\n0 0 lineto\n", length_y, length_x, length_y, length_x);
    fprintf(file, "closepath\nfill\n\n");

    fprintf(file, "newpath\n0 0 0 setrgbcolor\n");
    fprintf(file, "0 0 moveto\n0 %d lineto\n %d %d lineto\n%d 0 lineto\n0 0 lineto\n", length_y, length_x, length_y, length_x);
    fprintf(file, "closepath\nstroke\n\n");

//    fprintf(file, "%d %d translate\n", radius+border, radius+border);

    if(total == 0.0 || wedges != -1)
    {
        if (wedges == -1)
        {
            fprintf(file, "newpath\n1 1 1 setrgbcolor\n0 0 %d 0 360 arc\nclosepath fill\n",radius);
        }
        else
            fprintf(file, "newpath\n%s setrgbcolor\n0 0 %d 0 360 arc\nclosepath fill\n", lookup_color(wedges), radius);

        fprintf(file, "newpath\n0 0 0 setrgbcolor\n0 0 %d 0 360 arc\nclosepath stroke\n", radius);
        return;
    }

    for(i=0; i < size; i++)
    {
        double wedge_arc = 1.0*pie_data[i]/total*360;
//        fprintf(stderr, "WEDGE: %f\n", wedge_arc);
        fprintf(file, "newpath\n");
        fprintf(file, "%s setrgbcolor\n", lookup_color(i));
        fprintf(file, "0 0 %d %f %f arc\n", radius, current_arc, current_arc + wedge_arc);
        fprintf(file, "0 0 lineto\n");
        fprintf(file, "closepath fill\n\n");
        current_arc += wedge_arc;
    }

    fprintf(file, "newpath\n0 0 0 setrgbcolor\n0 0 %d 0 360 arc\nclosepath stroke\n", radius);
    fprintf(file, "%%%%EOF\n");
}

// Arguments:
//      filename - the name of the temporary file to write the postscript to
//      pie_data - a hashtable containing the data in terms of name, frequency
//                 pairs
void print_pie_chart_for_queue(char* filename, char* color, double traffic, int size, int* pie_data)
{
 //   assert(filename != NULL && pie_data != NULL);

    FILE* file = fopen(filename, "w"); 
    if(file == NULL)
    {
        fprintf(stderr, "ERROR!  %s.\n", strerror(errno));
        exit(0);
    }
    pie_chart_write(file, color, traffic, size, pie_data);
    fclose(file);
}

void print_graph_node_png(FILE* file, node_t* node)
{
    int i;
    char* color;
    assert(node != NULL && node->type == PNG_OP);
    if(!MATURE(node))
        color = queue_immature_color;
    else if(node_mergable(node))
        color = queue_mergable_color;
    else
        color = "black";

    fprintf(file, "    N%d_%d [label=\"N%d\\n", node->id, simtime, node->id);
    assert(node->data.pngs.png_size >= 0 &&
           node->data.pngs.png_size < MAX_PNGS);
    for(i=0; i < node->data.pngs.png_size; i++)
    {
        print_pngs(file, node->data.pngs.pngs[i]);
        fprintf(file, "\\n");
    }

    fprintf(file, "%d (%2.2f\%)\\n", node->count_cache[simtime % MAX_TIME],
                                     percent(node->count_cache[simtime % MAX_TIME],
                                           tick_packet_count));
    fprintf(file, "%d (%2.2f\%)", sum_max_simtime(node), percent(sum_max_simtime(node),
                                                           mature_packet_count()));
    fprintf(file, "\", color=%s]\n", color);
    fprintf(file, "    N%d_%d -> N%d_%d\n", node->id, simtime, node->left->id, simtime);
    fprintf(file, "    N%d_%d -> N%d_%d\n", node->id, simtime, node->right->id, simtime);
    if(config->verbose) if(node->parent != NULL) fprintf(file, "    N%d -> N%d\n", node->id, node->parent->id);
}

void print_graph_node_queue(FILE* file, node_t* node)
{
    char* shape;
    char* style;
    char* color;
    int* pie_data;

    if(node == node_default_find())
        style = queue_default_style;
    else
        style = queue_normal_style;

    if(!MATURE(node)) 
        color = queue_immature_color;
    else if(queue_splittable(node))
        color = queue_splittable_color;
    else if(queue_mergable(node))
        color = queue_mergable_color;
    else 
        color = queue_normal_color;

    p_stats totals;
    char*   pie_chart; 
    char  filename[256];
    snprintf(filename, 256, "/tmp/pie-n%d-%d.ps\0",  node->id, simtime);

/*
    p_stats_init(&totals);
    p_stats_total(&totals, node->data.queue.stats, MAX_TIME);
*/

    int radius;
//    float traffic = percent(sum_max_simtime(node), mature_packet_count());
    float traffic = percent(node->count_cache[simtime % MAX_TIME], tick_packet_count);
    
    if(traffic <= MAX_MERGE_RATE)
        radius = 20; // Perhaps use MIN_RATE here?
    else if(traffic >= MAX_RATE)
    {
        radius = 50;

        if(traffic >= MAX_RATE+10) radius = 60;
        if(traffic >= MAX_RATE+30) radius = 80;
        if(traffic >= 99) radius = 100;
    }
    else
        radius = 30;

    pie_chart = p_stats_print_pie_chart(&(node->data.queue.stats[simtime % MAX_TIME]), radius);
//    pie_chart = p_stats_print_pie_chart(&totals, radius);

    fprintf(file, "####! <BEGIN \"%s\">\n", filename);
    print_with_prefix(file, "#### ", pie_chart);
    fprintf(file, "####! <END>\n", filename);
    free(pie_chart);



    fprintf(file, "    N%d_%d [label=\"N%d\\n", node->id, simtime, node->id, node);
    fprintf(file, "%d (%2.2f\%)\\n", node->count_cache[simtime % MAX_TIME],
                                  percent(node->count_cache[simtime % MAX_TIME],
                                          tick_packet_count));
    fprintf(file, "%d (%2.2f\%)\\n%d\"", sum_max_simtime(node),
        percent(sum_max_simtime(node), mature_packet_count()), p_stats_count_protocols(&(node->data.queue.stats[simtime % MAX_TIME])));
//        percent(sum_max_simtime(node), mature_packet_count()), p_stats_count_protocols(&totals));

    fprintf(file, ", shape=epsf, shapefile =\"%s\", style=%s, fontcolor=%s]\n", filename, style, color);
}

void print_subtree(FILE* file, node_t* node)
{
    assert(node!=NULL);
    assert(node->type == PNG_OP || node->type == QUEUE);
    if(node->type == PNG_OP)
    {
        print_graph_node_png(file, node); 
        print_subtree(file, node->left);
        print_subtree(file, node->right);
    }
    else
        print_graph_node_queue(file, node);        
    fflush(file);
}

void print_graph(FILE* file)
{
    int i;
    fprintf(file, "digraph T%d {\n", simtime);
    fprintf(file, "    size=\"8,10\"");
    fprintf(file, "    node [fontsize=8];\n");

    fprintf(file, "    T%d [shape=plaintext, label=\"File %s\\n Time %d\\nQueues %d\\nLast %d Minutes %d\\nLast %d Minutes %d\\nTotal Packets %d\"]\n",
            simtime, config->files[datafile], simtime, num_queues, TICK_INTERVAL, tick_packet_count, 10*MAX_TIME, mature_packet_count(), total_packets);
    fprintf(file, "    T%d -> N%d_%d [ color = white]\n", simtime, node_head()->id, simtime);
    print_subtree(file, node_head());
    fprintf(file, "}\n");
}
