%option noyywrap

%{

#include"sim.h"
#include"flow.h"
#include"ihv_yacc.h"

#define MAX_IMPORT_STACK 256
YY_BUFFER_STATE import_stack[MAX_IMPORT_STACK];
int import_stack_size = 0;

char filename[4*1024];

YY_BUFFER_STATE current_buffer;
int lineno = 1;
void yyerror(char* s)
{
    fprintf (stderr, "ERROR: %d: %s at %s\n", lineno, s, yytext);
}
%}

%%

import      { current_buffer = YY_CURRENT_BUFFER; return IMPORT; }
export      { return EXPORT; }
globals     { return GLOBALS; }
show        { return SHOW; }
queues      { return QUEUES; }
monitor     { return MONITOR; }
replace     { return REPLACE; }
step        { return STEP; }
run         { return RUN; }
help        { return HELP; }
quit        { return QUIT; }
packets     { return PACKETS; }
flows       { return FLOWS; }
summary     { return SUMMARY; }
all         { return ALL; }
tcp         { return TCP; }
TCP         { return TCP; }
udp         { return UDP; }
UDP         { return UDP; }
with        { return WITH; }
head        { return HEAD; }
default     { return DEFAULT; }
"->"        { return ARROW; }
display     { return DISPLAY; }
to          { return TO; }
left        { return LEFT; }
right       { return RIGHT; }

[\n;]+   { if(yytext[0] == '\n')lineno++; return STMT_END; }
n[0-9]+   { yylval.val = atoi(yytext+1); return NODE_ID; } 
0x[A-Fa-f0-9]+  { yylval.val = (int)strtol(yytext, NULL, 16); return NUM; }
[0-9]+      { yylval.val = (int)strtol(yytext, NULL, 10); return NUM; }
[A-Za-z]+ { yylval.id = strcpy(filename, yytext); return FILENAME; }
\"(\\.|[^\\"])*\" {yylval.id = strcpy(filename, yytext); return STRING_LITERAL; }

[\[\]] { return yytext[0]; }
"|" { return yytext[0]; }
"." { return yytext[0]; }
"," { return yytext[0]; }
[\(\)] { return yytext[0]; }

[ \t\r]+    ;

<<EOF>>     {   if (import_stack_size > 0)
                {
                    YY_BUFFER_STATE buffer = import_stack[import_stack_size];
                    yy_delete_buffer(buffer);
                    yy_switch_to_buffer(import_stack[--import_stack_size]); 
                }
                else yyterminate();
            //    return STMT_END;
            } 
