%{
#include"sim.h"
#include"flow.h"
#include"interactive.h"
%}

/* literal keyword tokens */

%token GLOBALS SHOW MONITOR REPLACE STEP RUN HELP QUIT QUEUES STMT_END
%token PACKETS FLOWS SUMMARY ALL 
%token TCP UDP WITH HEAD DEFAULT EXPORT IMPORT STRING_LITERAL
%token LEFT RIGHT NOMATCH MATCH
%token ARROW TO DISPLAY FILENAME

%union
{
    char*       id;
    int         val;
    flow_t*     flow;
    node_t*     node;
    vector_t*   list;
    png_t*      png;
    png_op_t*   pnglist;
}

%token <id>  FILENAME
%token <id>  STRING_LITERAL
%token <val> NUM
%token <val> NODE_ID
%type <list> numlist
%type <val>  child
%type <node> node
%type <png>  png
%type <val>  addr
%type <val>  port
%type <val>  protocol
%type <flow> flow
%type <pnglist> pnglist
%%

stmtlist:       stmt
|               stmtlist stmt
;

stmt:           globals_cmd 
|               display_cmd 
|               export_cmd 
|               show_cmd 
|               import_cmd 
|               monitor_cmd 
|               replace_cmd 
|               QUEUES STMT_END      { queues_print(stdout); }
|               step_cmd 
|               RUN STMT_END         { command_run(); }
|               help_cmd
|               QUIT STMT_END        { command_quit(); }
;

globals_cmd:    GLOBALS STMT_END { print_params(stdout); }
;

//export_cmd:     EXPORT TO FILENAME STMT_END  { export_cmd($3); }
export_cmd:     EXPORT TO STRING_LITERAL STMT_END  { export_cmd($3); }
;

display_cmd:    DISPLAY STMT_END            { command_display_to_screen(); }
|               DISPLAY TO STRING_LITERAL STMT_END { command_display($3); }
|               DISPLAY node STMT_END       { printf("show node n%d on screen.\n", $2); }
|               DISPLAY node TO FILENAME STMT_END       { printf("print node n%d to %s.\n", $2, $4); }

import_cmd:     IMPORT STRING_LITERAL STMT_END  { if($2 != NULL) command_import($2); }

show_cmd:       SHOW node STMT_END         { if($2 != NULL) command_show($2, SHOW_SUMMARY); }
|               SHOW node PACKETS STMT_END { if($2 != NULL) command_show($2, SHOW_PACKETS); }
|               SHOW node FLOWS STMT_END   { if($2 != NULL) command_show($2, SHOW_FLOWS); }
|               SHOW node SUMMARY STMT_END { if($2 != NULL) command_show($2, SHOW_SUMMARY); }
|               SHOW node ALL STMT_END     { if($2 != NULL) command_show($2, SHOW_ALL); }
;

monitor_cmd:    MONITOR node STMT_END           { command_monitor($2, MONITOR_SUMMARY); }
|               MONITOR node PACKETS STMT_END   { command_monitor($2, MONITOR_PACKETS); }
|               MONITOR node FLOWS STMT_END     { command_monitor($2, MONITOR_FLOWS); }
|               MONITOR node SUMMARY STMT_END   { command_monitor($2, MONITOR_SUMMARY); }
|               MONITOR node ALL STMT_END       { command_monitor($2, MONITOR_ALL); }
|               MONITOR GLOBALS STMT_END        { globals = true; }
|               MONITOR flow STMT_END           { printf("monitoring flow: "); flow_print(stdout, $2); printf("\n"); flow_add($2); }
; 

replace_cmd:    REPLACE node WITH NODE_ID pnglist STMT_END  { if($2 == NULL)
                                                                  fprintf(stderr, "Unable to find node.\n");
                                                              else
                                                                  command_replace($2, $4, $5); 
                                                            }

step_cmd:       STEP STMT_END             { command_step(1); }
|               STEP NUM STMT_END         { command_step($2); }
;

help_cmd:       HELP STMT_END   { command_help(); }
;

flow:           protocol addr port ARROW addr port  { $$ = flow_new($1, $2, $5, $3, $6); }
;

protocol:       '(' NUM ')'  { $$ = $2; }
|               '(' TCP ')'  { $$ = 6;  }
|               '(' UDP ')'  { $$ = 17; }
;

addr:           NUM '.' NUM '.' NUM '.' NUM { $$ = $1 << 24 | $3 << 16 | $5 << 8  | $7; }
;

port:           '[' NUM ']'  { $$ = $2; }
;

node:           node '.' child {   if($$ != NULL)
                                       if($3 == true) { if($$->type == PNG_OP) $$ = $$->left; else $$ = NULL;  }
                                       else           { if($$->type == PNG_OP) $$ = $$->right; else $$ = NULL; }
                                   else
                                       $$ = NULL;
                               }

|               NODE_ID { $$ = node_find(node_head(), $1); if($$ == NULL) fprintf(stderr, "ERROR: n%d not found.\n", $1); }
|               HEAD    { $$ = node_head(); }
|               DEFAULT { $$ = node_default(); }
;

pnglist:        png                     { if($1 == NULL) $$ = NULL; png_op_t* pnglist = png_op_create(); png_op_add(pnglist, $1); $$ = pnglist; }
|               pnglist '|' png         { if($1 == NULL || $3 == NULL) $$ = NULL; png_op_add($1,$3); $$ = $1; }

png:            '(' NUM ',' numlist ')'  { vector_t* v = $4;
                                           if(v != NULL && v->size != N)
                                           {
                                               fprintf(stderr, "ERROR: png is of wrong size!\n");
                                               $$ = NULL;
                                           }
                                           else
                                           {
                                              png_t* ret = png_from_vector($2, $4);
                                              $$ = ret;
                                           }
                                         }
numlist:        NUM                 { $$ = vector_new($1); }
|               numlist NUM         { $$ = $1; vector_add($$, $2);  }
;

child:          LEFT                { $$ = true; }
|               RIGHT               { $$ = false; }
