// Interactive.c

#include<stdio.h>
#include<readline/readline.h>
#include<readline/history.h>
#include"constants.h"
#include"sim.h"
#include"ihv_lex.h"
#include"interactive.h"
#include"flow.h"

extern YY_BUFFER_STATE current_buffer;

static char* line_read = NULL;

typedef struct pair {
    char* command;
    char* params;
} pair_t;

static pair_t* split(char* line)
{
    int i;

    pair_t* pair = (pair_t*)malloc(sizeof(pair_t));
    pair->command = line;

    for(i=0; (line[i] != ' ' && line[i] != '\0'); i++);
    
    if(line[i] == ' ')
    {
        line[i] = '\0';
        pair->params = line+i+1;
    } 
    else
    {
        pair->params = NULL;
    }

    return(pair);
}

static char* rl_gets()
{
    char* ret;
    int length;
    if(line_read)
    {
        free(line_read);
        line_read = NULL;
    }

    line_read = readline("> ");

    if(line_read && *line_read)
    {
        add_history(line_read);
        length = strlen(line_read);
        ret = (char*)malloc(sizeof(char)*(length+2));
        memcpy(ret, line_read, sizeof(char)*length);
        ret[length] = '\n';
        ret[length+1] = '\0';
        line_read = ret;
    } 
    return(line_read);
}

void command_quit()
{
//    print_graph(stdout);
    exit(0);
}

void command_globals()
{
    print_params(stdout);
}

//void command_show(pair_t* pair)
void command_show(node_t* node, int flags)
{
    assert(node != NULL);

    switch(flags)
    {
        case SHOW_PACKETS:
            if(node->type == QUEUE)
                node_queue_print_packets(stdout, node); 
            else
                fprintf(stderr, "ERROR:  packets not available for PNG nodes.\n");
            break;
        case SHOW_FLOWS:
            if(node->type == QUEUE)
                node_queue_print_flows(stdout, node);
            else
                fprintf(stderr, "ERROR:  flows not available for PNG nodes.\n");
            break;
        case SHOW_SUMMARY:
            node_print(stdout, node);
            break;
        case SHOW_ALL:
            node_print(stdout, node);
            if(node->type == QUEUE)
            {
                node_queue_print_packets(stdout, node);
                node_queue_print_flows(stdout, node);
                node_print(stdout, node);
            }
            break;
        default:
            fprintf(stderr, "ERROR:  Unhandled option %d.\n", flags);
    }
}


void command_step(int ticks)
{
    int i;
    if(ticks < 0)
    {
        fprintf(stderr, "ERROR:  cannot move back in time!\n");
        return;
    }
    for(i=0; i < ticks;i++)
        if(datafile < config->numfiles)
            iterate();
        else
        {
            fprintf(stderr, "ERROR:  No more files to process.\n");
            break;
        }
}

void command_go(pair_t* pair)
{
    int ticks;
    node_t* node = NULL;
    if(pair->params == NULL)
    {
        fprintf(stderr, "ERROR:  format should be:  go <d>.\n");
        return;
    }

    ticks = atoi(pair->params);
    if(ticks > 0 )
    {
        int i;
        for(i=0; i < ticks;i++)
            if(datafile < config->numfiles)
                iterate();
            else
            {
                fprintf(stderr, "ERROR:  No more files to process.\n");
                break;
            }
    } 
}

void command_inspect(pair_t* pair)
{
    flow_t* flow = NULL;
    if(pair->params == NULL)
    {
        fprintf(stderr, "ERROR:  format should be:  inspect <flow>.\n");
        return;
    }

    flow = flow_from_string(pair->params);
    if(flow != NULL)
    {
        flow_add(flow); 
        fprintf(stdout, "Now inspecting ");
        flow_print(stdout, flow);
        fprintf(stdout, ".\n");
    }
    else
        fprintf(stderr, "ERROR:  format error in flow: %s.\n", pair->params);
}

void command_run()
{
    while(datafile < config->numfiles)
        iterate();
}

void command_monitor(node_t* node, int flags)
{
    if(node == NULL)
    {
        printlog(stdout, "node n%d not found.\n", node->id);
        return;
    }

    node->monitor = node->monitor | flags;
}

/*
void command_monitor(pair_t* pair)
{
    int i;
    node_t* node = NULL;
    if(pair->params == NULL || (pair->params[0] != 'n' && pair->params[0] != 'N'))
    {
        fprintf(stderr, "ERROR:  format should be:  show N<d>.\n");
        return;
    }

    i = atoi(++pair->params); 
    node = node_find(node_head(), i);
    if(node)
    {
        if(node->monitor)
        {
            node->monitor = false;
            fprintf(stdout, "No longer monitoring N%d.\n", node->id);
        }
        else
        {
            node->monitor = true;
            fprintf(stdout, "Monitoring N%d.\n", node->id);
        }
    }
    else
        fprintf(stderr, "ERROR: node N%d is no known.\n");
}
*/

void command_unknown(pair_t* pair)
{
    if(pair->command[0] == '\0') return;

    if(pair->params != NULL)
        fprintf(stderr, "ERROR: %s in \"%s %s\" is unknown.\n",
                        pair->command,
                        pair->command,
                        pair->params);
    else
        fprintf(stderr, "ERROR: \"%s\" is unknown.\n", pair->command);
}

void command_help()
{
    fprintf(stdout, "Commands:\n");
    fprintf(stdout, "globals                       - show values of the globals.\n");
    fprintf(stdout, "show n<d> [all|flows|packets] - show values of node n<d>.\n");
    fprintf(stdout, "queues                        - show summary values for all queues.\n");
    fprintf(stdout, "monitor                       - show values of node n<d> at each time period.\n");
    fprintf(stdout, "step <d>                      - jump ahead <d> ticks.\n");
    fprintf(stdout, "help                          - this message.\n");
    fprintf(stdout, "quit                          - quit the program.\n");
}

// Go forward one tick (file)
void command_continue()
{
    if(datafile < config->numfiles)
        iterate();
    else
    {
        fprintf(stdout, "No files to process.\n");
    }
}

/*
void command_do(char* command)
{
    pair_t* pair = NULL;
      
    if(command==NULL)                       // Ctrl-C
        command_quit();

    pair = split(command);

    if(strcmp(pair->command, "quit") == 0)              // QUIT
        command_quit();
    else if (strcmp(pair->command, "globals") == 0)     
        command_globals();
    else if (strcmp(pair->command, "show") == 0)
    { } //    command_show(pair); 
    else if (strcmp(pair->command, "go") == 0)
        command_go(pair);
    else if (strcmp(pair->command, "monitor") == 0)
        command_monitor(pair);
    else if (strcmp(pair->command, "inspect") == 0)
        command_inspect(pair);
    else if (strcmp(pair->command, "run") == 0)
        command_run();
    else if (strcmp(pair->command, "help") == 0)
        command_help();
    else if (strcmp(pair->command, "cont") == 0)        // Continue
        command_continue();
    else
        command_unknown(pair);
}
*/

void command_line()
{
    while(true)
    {
        char* command = rl_gets();
        if (command == NULL)
        {   printf("\n"); } 
        else if(strcmp(command, "") != 0)
        {
            YY_BUFFER_STATE buffer = NULL;
            int length = strlen(command);
            buffer = (YY_BUFFER_STATE)yy_scan_string(command);
            yy_switch_to_buffer(buffer);
            yyparse();
//            yy_delete_buffer(buffer);
        }
    }
}

/*
void command_file(FILE* file)
{
    char* command = NULL;
    size_t len = 0;
    ssize_t read;
    while((read = getline(&command, &len, file) != -1))
    {
        command_do(command);
    }
}
*/

/* IGNORING MEMORY MANAGEMENT HERE */
/*
void command_replace(node_t* node, int val, png_t* png)
{
    node_t* rep;
    node_t* old_id; 
    // Handle errors if we didn't find a node or if we can't
    // parse the png.
    if(node == NULL)
    {
        fprintf(stderr, "Invalid node.  Cannot replace node.\n");
        return;
    }
    else if (png == NULL)
    {
        fprintf(stderr, "Invalid png.  Cannot use this png.\n");
        return;
    }

    // Debug code.  Remove
    printf("Trying to replace n%d with n%d\n", node->id, val);
    printf("png: %d node %d\n", png, node->id);

    old_id = node_find(node_head(), val);
    if(old_id != NULL)
        old_id->id = id_get_next();
    
    rep = node_png_create_with_id(png, node->parent, val);

    if(node->parent == NULL)    // Is this HEAD?
        head_set(rep);
    else
    {
        node_update_child(node->parent, node, rep);
        // Should be freeing a bunch of nodes here.
    }
    rep->left   = node_queue_create(rep);
    rep->right  = node_queue_create(rep);
    // node_queue_free(node);
    //tree_export(stdout);
}
*/

void command_replace(node_t* old, int val, png_op_t* pnglist)
{
    node_t* rep;
    node_t* old_id = node_find(node_head(), val);

    assert(old != NULL && pnglist != NULL);

    if(old_id != NULL) old_id->id = id_get_next();

    rep = node_png_create_with_png_op_id(pnglist, old->parent, val);
    if(old->parent == NULL) head_set(rep);
    else {
        node_update_child(old->parent, old, rep);
        // Should be freeing a bunch of nodes here.
    }
    rep->left = node_queue_create(rep);
    rep->right = node_queue_create(rep);
} 

void export_cmd(char* filename)
{
/*
    char* tmp = (char*)malloc(sizeof(char)*strlen(filename));
    strcpy(tmp, filename);
*/
    char* tmp = copystr(filename);
    tmp++;
    tmp[strlen(tmp)-1] = '\0';
    FILE* fin = fopen(tmp, "w");
    free(--tmp);
    if(!fin)
    {
        fprintf(stderr, "Unable to create file: %s.\n", filename);
        return;
    }
    tree_export(fin);        
    fflush(fin);
    fclose(fin);
}


#define MAX_IMPORT_STACK 256
extern YY_BUFFER_STATE import_stack[MAX_IMPORT_STACK];
extern int import_stack_size;

void command_import(char* filename)
{
    YY_BUFFER_STATE buffer = NULL;
    FILE* fin;
    char* tmp = copystr(filename);

    tmp++; tmp[strlen(tmp)-1] = '\0';
    fin = fopen(tmp, "r");

    if(!fin)
    {
        fprintf(stderr, "Unable to open file: %s.\n", tmp);
        return;
    }

    if(import_stack_size >= MAX_IMPORT_STACK)
    {
        fprintf(stderr, "Imports nested too deeply\n");
        exit(1);
    }
    import_stack[import_stack_size++] = current_buffer;
    buffer = yy_create_buffer(fin, 16384);
    yy_switch_to_buffer(buffer);
    yyparse();
    regain_sanity();
}

void save_graph_to_file(char* filename) // This should go somewhere else
{
    FILE* fin = fopen(filename, "w");
    print_graph(fin);
    fflush(fin);
    fclose(fin);
}

void command_display(char* filename)
{
    char cmd[1024*4];
    save_graph_to_file(filename);
//    sprintf(cmd, "%s -Tps -Grotate=90 %s | %s - > %s.pdf", DOT_PATH, TMP_FILE, PS2PDF_PATH, filename);
//    sprintf(cmd, "%s -Tps -Grotate=90 %s > %s", DOT_PATH, filename);
//    system(cmd);
}

void command_display_to_screen()
{
    char cmd[1024*4];
    save_graph_to_file(TMP_FILE);
    sprintf(cmd, "%s -Tpng %s -o /tmp/graph.png", DOT_PATH, TMP_FILE);
    system(cmd);
//    system("eog /tmp/graph.png &");
    system("open -a Preview /tmp/graph.png");
}
