class MainController < ApplicationController
  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post, :only => [ :destroy, :create, :update ],
         :redirect_to => { :action => :list }

  def index
    setup_xhtml()

    @VERSION='1.0.2'  # The version to display on the webpage

    # version of the session data. you can use this to not load invalid
    # sessions if the stored session data changes structure.  then you won't
    # get a crash, you can just start a new session
    session[:version] = 1.0

    session[:tree] = "default" if !session[:tree]
    session[:navigation] = Navigation.new if !session[:navigation]
    if !session[:view]
      session[:view] = View.new(false)
      session[:view].duration = 10
    end

    load_view_revision
  end

  # the callback when the page is first shown
  def index_show_tree
    render_page(:update => :tree)
  end

  def zoom
    puts "zooming to #{params[:zoom]}"
    session[:navigation].zoom += params[:zoom].to_i
    render_page(:update => :navigation)
  end

  def pan_notify
    puts "panned to #{params[:x]}, #{params[:y]}"
    session[:navigation].viewx = params[:x].to_i
    session[:navigation].viewy = params[:y].to_i
    # nothing to update
    redirect_to :action => :none
  end

  # the callback when you want to show details for a node (on mouse-over)
  def node_details
    puts "details! for id #{params[:id]}"
    session[:details_id] = params[:id].to_i
    render_page(:update => :details)
  end

  def node_click
    puts "click"
    render_page(:update => :dialog, :details => { :id => params[:id] })
  end

  def dialog
    puts "dialog! for id #{params[:id]}"
    @svgnode = "hello"

#    tree = get_session_tree
#    tree.load_revision(session[:revision])
#    n = tree.find_node(params[:id].to_i)
#    puts "node #{n.id} #{n.terminal} #{n.position}"

    render :action => :dialog, :layout => false
  end

  def help
    render :action => :help, :layout => 'help'
  end

  # change what revisions are being shown (and make it in relative mode)
  def change_tree
    session[:tree] = params[:view_tree]
    redirect_to :action => 'refresh_tree'
  end

  def delete_tree
    if session[:tree] != 'default'
      t = Tree.find(:first, :conditions => "tree_name='#{session[:tree]}'")
      if t
        Tree.delete(t.id)
        session[:tree] = 'default'
      end
    end
    redirect_to :action => 'refresh_tree'
  end

  # change what revisions are being shown (and make it in relative mode)
  def change_time_range
    case params[:time_style]
    when 'absolute'
      puts "ABSOLUTE"
      session[:view].abs = true
    when 'relative'
      puts "RELATIVE"
      session[:view].abs = false
    end
    dur = params[:time_duration].to_i
    rel = params[:to_relative].to_i
    abs = params[:to_absolute].to_i

    puts "ABS? #{session[:view].abs}"
    session[:view].duration = dur
    session[:view].rel_to_time = rel
    session[:view].abs_to_time = Time.at(abs)
    redirect_to :action => 'refresh_tree'
  end

  def change_label
    c = if params[:classname]
          Classification.find_label(params[:classname])
        else nil
        end
    redirect_to(:action => :none) if !c

    redirect_to(:action => :none) if !params[:label_select]
    label_to = if params[:label_select].to_i == 0
                 nil
               else
                 l = Label.find params[:label_select].to_i
                 redirect_to(:action => :none) if !l
                 l
               end

    tree = get_session_tree
    tree.load_revision(session[:to_revision])
    n = if params[:id] then tree.find_node(params[:id].to_i) else nil end
    redirect_to(:action => :none) if !n

    Label.assign(tree.id, label_to, c, n)

    render_page :update => :tree
  end

  # show the correct tree
  def refresh_tree
    puts "refresh_tree"

    oldfrom = session[:from_revision]
    oldto = session[:to_revision]
    load_view_revision
    if oldfrom != session[:from_revision] || oldto != session[:to_revision]
      render_page(:update => :tree)
    else
      # just update small things
      render_page(:update => :refresh)
    end
  end

  def start_capture
    node = params[:id].to_i
    tree = get_session_tree

    req = CaptureRequest.start(tree, node) if (tree && node)
    if (req)
      puts "capturing node #{node}"
      render_page(:update => :capture)
    else
      redirect_to :action => :none
    end
  end

  def none
    render_page(:update => :none)
  end

  def get_capture
    id = params[:id]
    puts "retrieving capture id #{id}"

    t = Time.new
    node = CaptureRequest.get_node(id)
    filename = "dump-%04d-%02d-%02d.%02d.%02d-%s.pcap" %
      [t.year, t.month, t.day, t.hour, t.min, node]

    if id
      capdata = Capture.get(id) # get all the packets
      puts "capture data length: #{capdata.size}"
      send_data capdata, :filename => filename
    else
      redirect_to :action => index
    end
  end

  def end_capture
    id = params[:id]
    puts "done capturing id #{id}"

    if id
      CaptureRequest.end(id) # end the capturing (and delete from the db)
      render_page(:update => :capture)
    else
      redirect_to :action => :none
    end
  end

  def edit_labels
    @labels = Label.find(:all)
    render :update do |page|
      page.replace_html 'edit-labels-content', :partial => 'edit_labels'
      page << 'Element.hide("main-stuff");'
      page << 'Element.show("edit-labels");'
    end
  end

  def edit_labels_delete
    Label.delete(params[:id])
    @labels = Label.find(:all)
    render :update do |page|
      page.replace_html 'edit-labels-content', :partial => 'edit_labels'
      page << 'Element.hide("main-stuff");'
      page << 'Element.show("edit-labels");'
    end
  end

  def edit_labels_done
    render_page :update => :tree do |page|
      page << 'Element.hide("edit-labels");'
      page << 'Element.show("main-stuff");'
    end
  end

  def edit_labels_edit
    @label = Label.find(params[:id]) if params[:id]
    render :update do |page|
      page.replace_html 'edit-labels-content', :partial => 'edit_labels_edit'
      page << 'Element.hide("main-stuff");'
      page << 'Element.show("edit-labels");'
    end
  end

  def edit_labels_edit_done
    puts "saving label #{params[:id]}"
    l = if params[:id]
          Label.find(params[:id])
        else
          l = Label.new
          l.tree_id = get_session_tree.id
          l
        end
    l.name = params[:name]
    l.color = '#'+params[:red]+params[:green]+params[:blue]
    if l.save
      redirect_to :action => :edit_labels
    else
      # show the errors
      flash[:error] ||= ''
      l.errors.full_messages.each do |e| flash[:error] += e + '<br/>' end
      render :update do |page|
        page.replace_html 'edit_label_error_box', :partial => 'error_box'
      end
    end
  end

private
  def get_session_tree
    t = Tree.find(:first, :conditions => "tree_name='#{session[:tree]}'")
    if !t && session[:tree] != 'default'
      session[:tree] = "default"
      t = Tree.find(:first, :conditions => "tree_name='#{session[:tree]}'")
    end
    # XXX you can't select a tree yet, so when there is no tree present, just
    # make the default tree for now
    if !t
      t = Tree.new
      t.tree_name = session[:tree]
      t.save
    end
    t
  end

  def load_view_revision()
    # check what revision they should be looking at, from the available ones
    # and if it's not what they are looking at then switch to it

    tree = get_session_tree
    descrevs = Revision.find(:all,
                             :order => "revision DESC",
                             :conditions => "tree_id=#{tree.id}")
    ascrevs = Revision.find(:all,
                            :order => "revision ASC",
                            :conditions => "tree_id=#{tree.id}")
    # what times should we try to find revisions for?
    if session[:view].abs
      to = session[:view].abs_to_time
      puts "to #{to}"
    else
      rel_to = session[:view].rel_to_time

      to = Time.new
      to -= to.gmt_offset # get to in GMT time
      to -= rel_to * 60
    end
    duration = session[:view].duration
    from = to - duration * 60

    # find a revision for time 'from' and 'to'
    rev_to = rev_from = nil
    descrevs.each { |rev| rev_to = rev if !rev_to && rev.revision <= to }
    ascrevs.each { |rev| rev_from = rev if !rev_from && rev.revision >= from }

    if (rev_to && rev_from)
      session[:to_revision] = rev_to.id
      session[:from_revision] = rev_from.id
      puts "revision #{rev_from.revision.to_s} - #{rev_to.revision.to_s} is loaded"
    else
      puts "unable to load revisions for the time range"
      flash[:error] = "No available data for this range"
      session[:to_revision] = nil
      session[:from_revision] = nil
    end
  end

  def generate_time_ranges(revs)
    rel_times = {0 => 'Now',
      10 => '10 minutes ago',
      20 => '20 minutes ago',
      30 => '30 minutes ago',
      40 => '40 minutes ago',
      50 => '50 minutes ago',
      60 => '1 hour ago',
      90 => '1.5 hours ago',
      120 => '2 hours ago',
      180 => '3 hours ago',
      240 => '4 hours ago',
      300 => '5 hours ago',
      360 => '6 hours ago',
      480 => '8 hours ago',
      600 => '10 hours ago',
      720 => '12 hours ago',
      1440 => '1 day ago',
      2880 => '2 days ago',
      4320 => '3 days ago',
      5760 => '4 days ago',
      7200 => '5 days ago',
      8640 => '6 days ago',
      10080 => '1 week ago',
      20160 => '2 weeks ago',
      30240 => '3 weeks ago',
      40320 => '4 weeks ago'}
    durations = {10 => '10 minutes',
      20 => '20 minutes',
      30 => '30 minutes',
      40 => '40 minutes',
      50 => '50 minutes',
      60 => '1 hour',
      90 => '1.5 hours',
      120 => '2 hours',
      180 => '3 hours',
      240 => '4 hours',
      300 => '5 hours',
      360 => '6 hours',
      480 => '8 hours',
      600 => '10 hours',
      720 => '12 hours',
      1440 => '1 day',
      2880 => '2 days',
      4320 => '3 days',
      5760 => '4 days',
      7200 => '5 days',
      8640 => '6 days',
      10080 => '1 week',
      20160 => '2 weeks',
      30240 => '3 weeks',
      40320 => '4 weeks'}

    abs_times = {}
    revs.each do |i|
      seconds = i.revision.tv_sec
      time = Time.at(seconds)
      time += time.gmt_offset # convert to local time
      abs_times[seconds] = time.strftime("%b %d %Y, %I:%M %p")
    end

    [rel_times, abs_times, durations]
  end

  def render_page(args)
    tree = get_session_tree

    if args[:update] == :tree || args[:update] == :details
      puts "loading tree from db"
      tree.load_revision(session[:to_revision])
    end

    if args[:update] == :tree
      puts "loading traffic summaries"
      trafficsummary = {}
      begin
        fr = Revision.find(session[:from_revision])
        to = Revision.find(session[:to_revision])
      rescue ActiveRecord::RecordNotFound
        # nothin to do..
      else
        from_time = fr.revision
        to_time = to.revision
        trafficsummary = Traffic.summary_for_tree(tree, from_time, to_time)
      end
      session[:trafficsummary] = trafficsummary

      puts "building svg objects"
      tree.build_svg(trafficsummary)

      puts "layouting svg objects"
      w, h = tree.layout_svg
      puts "creating the navigation"
      session[:navigation].set_size(w, h)

      puts "rendering svg to xml"
      @svg, tree_root = tree.render_svg
      puts "rendering the navigation"
      session[:navigation].render_svg(@svg)
      puts "done rendering"

      puts "saving a list of the node ids"
      nodelist = {}
      termlist = {}
      def save_id(node, nodelist, termlist)
        return if !node
        nodelist[node.position] = node.node_id.to_s
        termlist[node.position] = node.node_id.to_s if node.terminal
        save_id(node.left, nodelist, termlist)
        save_id(node.right, nodelist, termlist)
      end
      save_id(tree.root, nodelist, termlist)
      session[:nodelist] = nodelist
    end

    if args[:update] == :tree || args[:update] == :refresh
      puts "getting the available revisions"

      revs = Revision.find(:all,
                           :select => 'revision',
                           :conditions => "tree_id=#{tree.id}")
      @relative_times, @absolute_times, @durations =
        generate_time_ranges(revs)

      # set these for the rendering templates
      @relative_time_selected = session[:view].rel_to_time
      @absolute_time_selected = session[:view].abs_to_time.tv_sec
      @duration_selected = session[:view].duration

      puts "getting the available trees"

      @tree_names = []
      trees = Tree.find(:all)
      trees.each do |i|
        @tree_names << i.tree_name
      end
      @tree_selected = session[:tree]
    end

    nav = session[:navigation]
    nodelist = session[:nodelist]

    if args[:update] == :tree || args[:update] == :details
      puts "building the node details"
      build_node_details(tree)
    end

    puts "rendering time..."
    render :update do |page|
      if args[:update] == :tree || args[:update] == :times
        page.replace_html 'tree_form', :partial => 'tree_form'
      end

      if args[:update] == :times
        page.replace_html 'to_absolute', :partial => 'absolute_time'
      end

      if args[:update] == :tree
        page.replace_html 'svg-tree', @svg.to_s
        if to_time
          t = to_time + to_time.gmt_offset # get local time
          page.replace_html 'generate-time', 'This tree generated on ' +
            t.strftime('%b %d, %Y') + ' at ' +
            t.strftime('%I:%M %p')
        else
          page.replace_html 'generate-time', ''
        end
        page.replace_html 'time_form', :partial => 'time_form'

        if session[:view].abs
          page << 'Element.show("absolute_time_select");'
          page << 'Element.hide("relative_time_select");'
          page << '$("absolute_time_check").checked = true;'
        else
          page << 'Element.hide("absolute_time_select");'
          page << 'Element.show("relative_time_select");'
          page << '$("relative_time_check").checked = true;'
        end
        page << '$("absolute_time_check").onclick = function ()   '+
          '{                                                      '+
          '  Element.show("absolute_time_select");                '+
          '  Element.hide("relative_time_select");                '+
          '}                                                      '
        page << '$("relative_time_check").onclick = function ()   '+
          '{                                                      '+
          '  Element.hide("absolute_time_select");                '+
          '  Element.show("relative_time_select");                '+
          '}                                                      '

        # window resize handler
        page <<
          "function resizeHandler() {                                  "+
          "  var BODY_MARGINS = 10;  /* top and bottom body margin */  "+
          "  var TREE_PADDING = 6;   /* padding above and below the tree picture */"+
          "  var area = clientHeight();                                "+
          "  var title = getElementHeight('titlebar');                 "+
          "  var time = getElementHeight('generate-time');             "+
          "  var h = area - title - time;                              "+
          "  h = h - BODY_MARGINS;                                     "+
          "  h = h - TREE_PADDING;                                     "+
          "  $('svg-picture-root').setAttribute('height', h);          "+
          "}                                                           "+
          "window.onresize = resizeHandler;                            "+
          "resizeHandler();                                            ";

        # drag handler
        page <<
          "var svgDragInProgress = false;                              "+

          "function svgDrag(e) {                                       "+
          "  myStopPropagation(e);                                     "+
          "  var dx = svgDragX - e.clientX;                            "+
          "  var dy = svgDragY - e.clientY;                            "+
          "  svgDragX = e.clientX;                                     "+
          "  svgDragY = e.clientY;                                     "+
          "  var areaWidth = $('svg-tree-root').width.baseVal.value;   "+
          "  var areaHeight = $('svg-tree-root').height.baseVal.value; "+
          "  var px = dx / areaWidth;                                  "+
          "  var py = dy / areaHeight;                                 "+
          "  nowViewX += px * nowTotalWidth / nowZoom;                 "+
          "  nowViewY += py * nowTotalHeight / nowZoom;                "+
          "  nowViewX = (nowViewX < minViewX ? minViewX :              "+
          "              (nowViewX > maxViewX ? maxViewX : nowViewX)); "+
          "  nowViewY = (nowViewY < minViewY ? minViewY :              "+
          "              (nowViewY > maxViewY ? maxViewY : nowViewY)); "+
          "  $('svg-tree-root').setAttribute('viewBox',                "+
          "    ''+nowViewX+' '+nowViewY+' '+                           "+
          "       nowViewWidth+' '+nowViewHeight);                     "+
          "}                                                           "

        # drag ender
        page <<
          "function svgDragEnd(e) {                                     "+
          "  myStopPropagation(e);                                      "+
          "  if (isLeftButton(e)) {                                     "+
          "    svgDrag(e);                                              "+
          "    svgDragInProgress = false;                               "+
          "    document.body.style.cursor = 'default';                  "+
          "    window.onmousemove = null;                               "+
          "    window.onmouseup = null;                                 "+
          "    /* notify the server so it can save it in the session */ "+
          "    new Ajax.Request('/main/pan_notify?'+                    "+
          "                     'x='+nowViewX+'&y='+nowViewY,           "+
          "                     {asynchronous:true, evalScripts:true}); "+
          "  }                                                          "+
          "}                                                            "

        # drag starter
        page << "$('svg-tree').onmousedown =                         "+
          "function (e) {                                            "+
          "  if (isLeftButton(e)) {                                  "+
          "    myStopPropagation(e);                                 "+
          "    svgDragInProgress = true;                             "+
          "    /*window.onmousemove = svgDrag;*/                     "+
          "    window.onmouseup = svgDragEnd;                        "+
          "    document.body.style.cursor = 'crosshair';             "+
          "    svgDragX = e.clientX;                                 "+
          "    svgDragY = e.clientY;                                 "+
          "  }                                                       "+
          "}                                                         "

        page << "var mouseOverTimeout = null;"

        nodelist.each do |pos, id|
          page << "function mouse_over_#{id}()                       "+
            "{                                                       "+
            "  showBusySpinner();                                    "+
            "  new Ajax.Request('/main/node_details?id=#{id}',       "+
            "                   {asynchronous:true, evalScripts:true,"+
            "                    onComplete:function(request)        "+
            "                      {hideBusySpinner();}});           "+
            "}                                                       "
          page << "$('#{id}').onmouseover =                              "+
            "function (e) {                                              "+
            "  if (svgDragInProgress) return;                            "+
            "  if (mouseOverTimeout) {                                   "+
            "    clearTimeout(mouseOverTimeout);                         "+
            "  }                                                         "+
            "  mouseOverTimeout = setTimeout(mouse_over_#{id}, 500);     "+
            "}                                                           "
          page << "$('#{id}').onmouseout =                               "+
            "function (e) {                                              "+
            "  if (mouseOverTimeout) {                                   "+
            "    clearTimeout(mouseOverTimeout);                         "+
            "    mouseOverTimeout = null;                                "+
            "  }                                                         "+
            "}                                                           "
        end
        # you can only capture packets on terminal nodes right now
        termlist.each do |pos, id|
          page << "$('#{id}').onmousedown =                              "+
            "function (e) {                                              "+
            "  if (controlPressed(e)) {                                  "+
            "    myStopPropagation(e);                                   "+
            "    showBusySpinner();                                      "+
            "    new Ajax.Request('/main/start_capture/#{id}',           "+
            "                     {asynchronous:true, evalScripts:true,  "+
            "                      onComplete:function(request)          "+
            "                        {hideBusySpinner();}});             "+
            "  }                                                         "+
            "}                                                           "
        end
      end

      if args[:update] == :tree || args[:update] == :navigation
        page << "minViewX = #{nav.min_viewx};"
        page << "minViewY = #{nav.min_viewy};"
        page << "maxViewX = #{nav.max_viewx};"
        page << "maxViewY = #{nav.max_viewy};"
        page << "nowViewX = #{nav.viewx};"
        page << "nowViewY = #{nav.viewy};"
        page << "nowViewWidth = #{nav.viewwidth};"
        page << "nowViewHeight = #{nav.viewheight};"
        page << "nowTotalWidth = #{nav.totalwidth};"
        page << "nowTotalHeight = #{nav.totalheight};"
        page << "nowZoom = #{nav.zoom};"

        page << "$('zoom-in').style.cursor = 'pointer';"
        page << "$('zoom-in').onmousedown =                            "+
          "function (e) {                                              "+
          "  myStopPropagation(e);                                     "+
          "  new Ajax.Request('/main/zoom?zoom=1',                     "+
          "                   {asynchronous:true, evalScripts:true});  "+
          "}                                                           "
        page << "$('zoom-out').style.cursor = 'pointer';"
        page << "$('zoom-out').onmousedown =                           "+
          "function (e) {                                              "+
          "  myStopPropagation(e);                                     "+
          "  new Ajax.Request('/main/zoom?zoom=-1',                    "+
          "                   {asynchronous:true, evalScripts:true});  "+
          "}                                                           "

        page <<
          "$('svg-tree-root').setAttribute('viewBox',                "+
          "  ''+nowViewX+' '+nowViewY+' '+                           "+
          "     nowViewWidth+' '+nowViewHeight);                     "
      end

      if args[:update] == :tree || args[:update] == :details
        page.replace_html 'mouseoverinfo', :partial => 'node_details'
      end

      if args[:update] == :capture || args[:update] == :tree ||
          args[:update] == :refresh
        @captures = {}
        CaptureRequest.current(tree).each do |c|
          @captures[c.id] = { :node => c.node_id,
                              :count => Capture.get_size(c.id) }
        end
        if !@captures.empty?
          page.replace_html 'capture-status', :partial => 'capture_status'
          page << "Element.show('capture-status');"
        else
          page << "Element.hide('capture-status');"
        end
      end

      if args[:update] == :dialog
        id = args[:details][:id]
        puts "#{id} !"
        page.call('window.open',
                  "/main/dialog?id=#{id}",
                  'Node details',
                  "status=0,toolbar=0,location=0,menubar=0,scrollbars=1," +
                  "height=400,width=400")
      end

      # always display any errors
      page.replace_html 'error_box', :partial => "error_box"

      # run any code block that got passed to us last
      yield page if block_given?
    end
  end

  def build_node_details(tree)
    return if !session[:details_id]
    return if !session[:trafficsummary]

    summary = session[:trafficsummary][session[:details_id]]
    return if !summary

    n = tree.find_node(session[:details_id])
    return if !n

    user_labels = Label.find_for_node(tree.id, n)

    @node_id = session[:details_id]
    @labels = Label.find_all(tree.id) if n.terminal
    @classes = {}
    summary.names.each do |name|
      label = user_labels[name]
      realname, color = if label
                          [label.name, label.color]
                        else
                          c = Classification.find_label(name)
                          [c.label, c.color]
                        end
      ck = ColorKey.new(realname, color)
      ck.build_svg
      @classes[name] = { :amount => summary.amount(name),
                         :percent => summary.percent(name),
                         :colorkey => ck.render_svg,
                         :label => label}
    end
    @total = summary.total
  end
end
