# A ColorKey is a non-database-model.  It represents a colored box rendered
# with SVG to display a color in a legend
class ColorKey
  # The size of ColorKey SVG boxes
  COLORKEY_SIZE = 20

  # Create a new ColorKey
  # @param name The name associated with this color, which is shown in a
  #             tooltip
  # @param colorstring The color to display inside the ColorKey's box, in web
  #                    color format, e.g. #AA06B2
  def initialize(name, colorstring)
    @name = name
    @cs = colorstring
  end

  # Build the SVG object for the ColorKey
  def build_svg()
    @svg = SVG::ColorKey.new(COLORKEY_SIZE, SVG::Color.new(@cs))
    @svg.tooltip = @name
  end

  # Render the SVG object to XML
  # @return Returns a libXML <svg> element which is the svg for displaying the
  #         ColorKey
  def render_svg
    doc = XML::Document.new
    doc.root = XML::Node.new('svg')
    doc.root['class'] = 'svg-color-key'
    doc.root['xmlns'] = 'http://www.w3.org/2000/svg'
    doc.root['xmlns:xlink'] = 'http://www.w3.org/2000/xlink/namespace/'
    doc.root['version'] = '1.1'
    doc.root['width'] = COLORKEY_SIZE.to_s
    doc.root['height'] = COLORKEY_SIZE.to_s

    doc.root.child = @svg.to_xml
    doc.root
  end
end
