#!/bin/sh

# NOTE: The version is dictated by the backend's configure.ac file

quit() {
  m="Error occured during execution. Exiting."
  if test -n "$1"; then m=$1; fi
  echo $m
  echo
  exit 1
}

distname="netadhict"         # name of the distribution

bdir=backend                 # where to find the backend code
bdistdir=backend             # the subdir for the backend in the distribution
fdir=frontend                # where to find the frontend code
fdistdir=frontend            # the subdir for the frontend in the distribution

# The version of the distribution
version="$(grep AC_INIT\( ./$bdir/configure.ac | \
           cut -d '[' -f 3 | cut -d ']' -f 1)"
bname="$(grep AC_INIT\( ./$bdir/configure.ac | \
         cut -d '[' -f 2 | cut -d ']' -f 1)"

btardir="$bname-$version"    # the directory extracted from the backend tar
btar="$btardir.tar.gz"       # the name of the tar built by the backend

wdir="$distname-$version"    # the working directory to build the distribution
disttar="$wdir.tar.gz"       # the name of the final tar for distribution

# Stuff from the toplevel dir to include in the distribution
DIST_FILES="README HACKING INSTALL LICENSE CHANGELOG makedist.sh"

# The stuff to distribute from the frontend component
FRONTEND_DIST_COPY="app config db doc lib public Rakefile README script test"
# Create these extra directories in the frontend distribution
FRONTEND_DIST_EMPTY="log"

test -e backend/Makefile || \
  quit "ERROR: Cannot find backend/Makefile.\nRun configure in the backend module before using this script."

test -e ./makedist.sh || \
  quit "ERROR: Run this script from the toplevel directory of netadhict."

test -e $wdir && \
  quit "ERROR: cannot create directory \"$wdir\" as there is something in the way."

set -x
cd $bdir                              || quit
make dist                             || quit
cd ..                                 || quit
mkdir $wdir                           || quit
mv $bdir/$btar $wdir                  || quit
cd $wdir                              || quit
tar xfz $btar                         || quit
mv $btardir $bdistdir                 || quit
rm $btar                              || quit
mkdir $fdistdir                       || quit
for i in $FRONTEND_DIST_COPY; do
  cp -a ../$fdir/$i $fdistdir         || quit
done
for i in $FRONTEND_DIST_EMPTY; do
  mkdir $fdistdir/$i                  || quit
done
cd ..                                 || quit
for i in $DIST_FILES; do
  cp -a $i $wdir                      || quit
done
tar cfz $disttar $wdir                || quit
rm -r -f $wdir                        || quit
